package cn.com.qmth.stmms.api.controller.admin;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import javax.servlet.http.HttpServletResponse;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;

import com.qmth.boot.core.collection.PageResult;
import com.qmth.boot.core.exception.StatusException;

import cn.com.qmth.stmms.admin.dto.OperationLogDTO;
import cn.com.qmth.stmms.api.controller.BaseApiController;
import cn.com.qmth.stmms.biz.exam.model.OperationLog;
import cn.com.qmth.stmms.biz.exam.query.OperationLogSearchQuery;
import cn.com.qmth.stmms.biz.exam.service.OperationLogService;
import cn.com.qmth.stmms.biz.utils.PageUtil;
import cn.com.qmth.stmms.common.annotation.Logging;
import cn.com.qmth.stmms.common.domain.ApiUser;
import cn.com.qmth.stmms.common.enums.LogType;
import cn.com.qmth.stmms.common.utils.ExportExcel;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

@Api(tags = "操作日志")
@Controller("adminOperationLogController")
@RequestMapping("/api/admin/operation/log")
public class OperationLogController extends BaseApiController {

    protected static Logger log = LoggerFactory.getLogger(OperationLogController.class);

    @Autowired
    private OperationLogService logService;

    @ApiOperation(value = "日志列表")
    // @Logging(menu = "日志查询", type = LogType.QUERY)
    @RequestMapping(value = "query", method = RequestMethod.POST)
    @ResponseBody
    public PageResult<OperationLog> query(OperationLogSearchQuery query) {
        ApiUser user = getApiUser();
        int examId = getSessionExamId();
        query.setExamId(examId);
        query.setSchoolId(user.getUser().getSchoolId());
        query.orderByCreateTime();
        query = logService.findByQuery(query);
        return PageUtil.of(query);
    }

    @ApiOperation(value = "日志类型")
    @RequestMapping(value = "types", method = RequestMethod.POST)
    @ResponseBody
    public List<LogType> examTypeList() {
        return Arrays.asList(LogType.values());
    }

    @ApiOperation(value = "导出日志")
    @Logging(menu = "导出日志", type = LogType.EXPORT)
    @RequestMapping(value = "/export", method = RequestMethod.POST)
    @ResponseBody
    public void exportFile(OperationLogSearchQuery query, HttpServletResponse response) {
        try {
            String fileName = "日志数据.xlsx";
            ApiUser user = getApiUser();
            int examId = getSessionExamId();
            query.setExamId(examId);
            query.setSchoolId(user.getUser().getSchoolId());
            query.orderByCreateTime();
            query = logService.findByQuery(query);
            List<OperationLogDTO> list = new ArrayList<>();
            for (OperationLog log : query.getResult()) {
                OperationLogDTO vo = new OperationLogDTO(log);
                list.add(vo);
            }
            new ExportExcel("日志数据", OperationLogDTO.class).setDataList(list).write(response, fileName).dispose();
        } catch (Exception e) {
            log.error(e.getMessage());
            e.printStackTrace();
            throw new StatusException("导出日志数据失败！" + e.getMessage());
        }
    }

}
