package cn.com.qmth.stmms.api.controller.admin;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import com.qmth.boot.core.collection.PageResult;

import cn.com.qmth.stmms.api.controller.BaseApiController;
import cn.com.qmth.stmms.biz.basic.bean.RoleInfoQuery;
import cn.com.qmth.stmms.biz.basic.bean.RoleInfoVo;
import cn.com.qmth.stmms.biz.basic.bean.RolePrivilegeDomain;
import cn.com.qmth.stmms.biz.basic.bean.RolePrivilegeVo;
import cn.com.qmth.stmms.biz.basic.model.Privilege;
import cn.com.qmth.stmms.biz.basic.service.PrivilegeService;
import cn.com.qmth.stmms.biz.basic.service.RoleInfoService;
import cn.com.qmth.stmms.biz.basic.service.RolePrivilegeService;
import cn.com.qmth.stmms.biz.exam.bean.ResultMessage;
import com.qmth.boot.core.exception.StatusException;
import cn.com.qmth.stmms.biz.utils.PageUtil;
import cn.com.qmth.stmms.common.domain.ApiUser;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

@Api(tags = "角色权限")
@Controller
@RequestMapping("/api/admin/role")
public class RoleController extends BaseApiController {

    @Autowired
    private RoleInfoService roleInfoService;

    @Autowired
    private RolePrivilegeService rolePrivilegeService;

    @Autowired
    private PrivilegeService privilegeService;

    @ApiOperation(value = "角色分页查询")
    @ResponseBody
    @RequestMapping(value = "page", method = RequestMethod.POST)
    public PageResult<RoleInfoVo> getPage(RoleInfoQuery query) {
        if (query.getSchoolId() == null) {
            throw new StatusException("SchoolId不能为空");
        }
        Integer totalCount = roleInfoService.countByQuery(query);
        query.setTotalCount(totalCount);
        if (totalCount > 0) {
            List<RoleInfoVo> list = roleInfoService.findByQuery(query);
            return PageUtil.of(list, query);
        } else {
            return PageUtil.emptyPage();
        }
    }

    @ApiOperation(value = "角色下拉列表")
    @ResponseBody
    @RequestMapping(value = "list/all", method = RequestMethod.POST)
    public List<RoleInfoVo> getAll(@RequestParam Integer schoolId) {
        return roleInfoService.allList(schoolId);
    }

    @ApiOperation(value = "当前用户权限code集合")
    @ResponseBody
    @RequestMapping(value = "privilege/codes", method = RequestMethod.POST)
    public Set<String> privilegeCodes() {
        ApiUser wu = getApiUser();
        return rolePrivilegeService.getRoleCodes(wu.getUser().getSchoolId(), wu.getRole());
    }

    @ApiOperation(value = "查询角色权限树")
    @ResponseBody
    @RequestMapping(value = "privilege/all", method = RequestMethod.POST)
    public List<RolePrivilegeVo> privilegeAll(RoleInfoQuery query) {
        if (StringUtils.isBlank(query.getRole())) {
            throw new StatusException("role不能为空");
        }
        if (query.getSchoolId() == null) {
            throw new StatusException("SchoolId不能为空");
        }
        return rolePrivilegeService.findByQuery(query);
    }

    @ApiOperation(value = "当前用户左侧菜单树")
    @ResponseBody
    @RequestMapping(value = "privilege/menu", method = RequestMethod.POST)
    public List<Privilege> privilegeMenu() {
        ApiUser wu = getApiUser();
        List<Privilege> ps = privilegeService.getMenuPrivileges();
        Set<String> codes = rolePrivilegeService.getRoleCodes(wu.getUser().getSchoolId(), wu.getRole());
        List<Privilege> ret = new ArrayList<>();
        for (Privilege p : ps) {
            if (codes.contains(p.getCode())) {
                ret.add(p);
            }
        }
        return ret;
    }

    @ApiOperation(value = "保存权限")
    @ResponseBody
    @RequestMapping(value = "privilege/save", method = RequestMethod.POST)
    public ResultMessage privilegeSave(RolePrivilegeDomain domain) {
        if (StringUtils.isBlank(domain.getRole())) {
            throw new StatusException("role不能为空");
        }
        if (domain.getSchoolId() == null) {
            throw new StatusException("SchoolId不能为空");
        }
        ApiUser wu = getApiUser();
        domain.setUserId(wu.getUser().getId());
        rolePrivilegeService.save(domain);
        return resultOk();
    }
}
